#pragma rtGlobals=1		// Use modern global access method.

//ELLIS 2 FUNCTION TEMPLATE

//Menu Items
//makes it easy to run your setup function.  Just edit the Submenu to add your function.  I have included a sample function, "MyFunction"
Menu "Ellis2" 
	Submenu "1. Run your setup function"
		"1.C Setup MyFunction", setupMyFunction() //menu entry 
	End
End

//Setup Function
//Every function requires a setup function.  It tells ellis what the initial limits, coefficients, temporary waves, etc. should be for your custom function.
Function setupMyFunction() //This is an impedance function for a randles cell.  As such, there are 3 coefficients, RS, Cdl and RP.  
	Make/T/O coeffnames={"RS","Cdl","RP"}	// make a list of coefficient names
	Make/O/D coefs = {100,0.000001,3e3}			// make the initial coefficients
	Make/O/D limits ={{0,0,0},{1e3,0.1,1e8}}		// make the lower and upper limits.  Make sure that coefs fall within these limits, or you will have problems.
	Make/O/I/N=(numpnts(coefs)) heldlist=0		// Makes the held list; decides whether or not each coefficient should be held.  0 = not held.  1 = held. (during fit).  Using this line, EVERYTHING will vary unless dictated by the user.
	//Make/O/I/N=(numpnts(coefs)) heldlist={1,0,0} //use this line instead of the above line if you need to specify coefficients that should ALWAYS be held.  In this example, I have set RS to be held by default.
	string/g currentmodel="Zrandles"
	make/t/o tempwavenames={"complexer"} //Declare any temporary waves you need.  ALL functions since Ellis 2.2 REQUIRE "complexer".  You may add extra entries to this list and the appropriate temporary complex waves will be made during preflight.
	preflight1() //Don't edit this.  This calls the preflight that processes everything above and makes ellis ready to do a fit.  All setup functions are required to call preflight1.
End

//Impedance Function
//Define your impedance function.  You must define a complex impedance function, not "parts 1 and 2"

Threadsafe Function MyFunction(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw//declare wave names used by default.  Don't edit this
	wave/c/d complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.  If you have made a proper setup function, this will be handled automatically.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.  Do not edit this.
	variable/g/c ki//pull in complex constant  Do not edit this.  You can also pull in kii, which is equal to (1-j)
	//variable/g/c/kii //Optional.  See above.
	variable Rs, Rp, Cdl //declare variable names; you can edit this.  For sanity's sake, these should match coeffnames from your setup function.
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Cdl=w[1]
	Rp=w[2]

	// Calculate your function here.  Make sure to keep track of temporary waves and variables.  Each line in this section should (in general) begin with a temporary wave name as defied in tempwavenames in your setup function., however you can do more complicated things here as well.  
	//This example is simple enough to be computed on one line, so we do not need to use this section.

	complexer[0,midpointt-1] = Rs+1/(1/Rp+1/(ki/-(Cdl*freqw)))  //Compute the final value of the complex function here.
	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
End