#pragma rtGlobals=1		// Use modern global access method.
#include <KBColorizeTraces>
#include <Wave Arithmetic Panel>
#include <TintedWindowBackground>

//Ellis 2.2 Beta 1 (cc) Matthew Taylor 2012
//Requires you to write your own fit function.  Some sample functions are included in Functions.ipf.
//The format of the function is an all-at-once complex function.  See Functions.ipf for further details.

//Menus
Menu "Ellis2" //gives menu
	"(follow directions. "
	"0.  Paste your data here.", Preflight0() //only needs to be run once, initializing the system.
		
	Submenu "Frequency conversion"
		"Hz -> angular", Execute "hz2ang("+testsetupvals[4]+")"
		"angular -> Hz", Execute "ang2hz("+testsetupvals[4]+")"
	End	
	Submenu "1. Run your setup function" //placeholder for function library.  Submenu contents are contained in Ellis2FunctionLibrary.ipf
	
	End
	Submenu "1. Run your setup function" //placeholder for function library.  Submenu contents are contained in Ellis2FunctionLibrary.ipf
	End
	"2. Display tables and graphs", onscreen()
	"3. Fit", DAfit()
	"Y. Preview Function", Simulatefunc()
	"Z. Toggle Synthetic / Data", Loadsimdata()
	"A. run error stack", stackError(algsetupvals[10],0,0)
	Submenu "B. Confidence Bands"
		"Compute Confidence Bands", RunConfidenceBands()
		"Display C-band results",RunDisplayConfidenceBands()
		"Display c-band edges", displayconfidenceedges()
	End
End

//Utilities
Macro preflight0() //after function setup has been run, this macro generates all of the tables and waves needed to do the fit.
	setdatafolder root:
	newdatafolder/o root:ellis
	newdatafolder/o root:ellis:onDeck
	newdatafolder/o root:ellis:batch
	setdatafolder root:ellis:onDeck
	Make/O/T algsetupnames = {"Method","Quiet","Population","crossover","Mutator","seed","Iterations", "Strategy","UNUSED","Tolerance","Trials","Trial Number","Userfunc?"}
	Make/O/D algsetupvals ={0,1,20,0.8,0.8,0,100,0,0,1e-7,0,0,0}
	Make/O/T testsetupnames = {"Model Name","batch folder","Data Z1", "Data Z2","Data w","Experiment Subfolder","User Function Name"}
	Make/O/T testsetupvals = {"model","batch0","z1data","z2data","wdata","experiment0","userfunc"}
	make/o/d/n=1 z1data=0
	make/o/d/n=1 z2data=0
	make/o/d/n=1 wdata=0
	make/o/d/n=1 sim_real=0
	make/o/d/n=1 sim_imag=0
	make/o/d/n=1 mask=0
	make/o/d/n=1 weight=0
	dataTable()
	algsetup()
	Print "Paste your data into the table, make adjustments to the algorithm parameters and load the data."
End

Function preflight1()
string/g currentmodel
	wave/t testsetupvals //post-script
	testsetupvals[0]=currentmodel
	wave/d algsetupvals
	wave/t testsetupvals, tempwavenames
 	make/n=(algsetupvals[6])/d/o Chi2prog = 0
 	SetScale/P x 0,algsetupvals[2]*numpnts(coefs),"", Chi2prog
	make/d/n=(numpnts(coefs), algsetupvals[6])/o pars = 0
	SetScale/P x 0,algsetupvals[2],"", pars
	Execute "duplicate/o "+ testsetupvals[2] + " realdata" //load data into toaster
	Execute "duplicate/o "+ testsetupvals[3] + " imagdata"
	Execute "duplicate/o "+ testsetupvals[4] + " freqdata"
	variable/g midpointt =numpnts(freqdata)
	maketempwaves(tempwavenames)
	concatenate/np/o/d {freqdata,freqdata}, freqdataa
	concatenate/np/o/d {realdata,imagdata}, combdataa
	make/o/d/n=(numpnts(freqdata)) ratiodata
	wave realdata, imagdata, freqdata
	ratiodata=realdata/imagdata
	make/o/d/n=(numpnts(freqdata)) modulusdata
	duplicate/o modulusdata fit_modulus
	make/d/o/n=(numpnts(freqdata)) phasedata
	duplicate/o phasedata fit_phase
	modulusdata=(realdata^2+imagdata^2)^(1/2)
	duplicate/o realdata res_real
	duplicate/o imagdata res_imag
	duplicate/o modulusdata res_modulus
	phasedata=180/(Pi)*atan(imagdata/realdata)
	duplicate/o phasedata res_phase
	make/d/o/n=(numpnts(freqdataa)) displayfit
	simulatefunc()
	Execute "timegraph()"
End

Function ang2hz(spectrum)
	wave spectrum
	spectrum/=2*Pi
End

Function hz2ang(spectrum)
	wave spectrum
	spectrum*=2*Pi
End

Function maketempwaves(namelist)
	wave/t namelist
	variable/g midpointt
	variable tempnamecount = numpnts(namelist)+1
	variable ii
	for(ii = 1 ; ii < tempnamecount ; ii += 1)
		Execute "make/c/o/d/n=(midpointt) "+namelist[ii-1]+"=0"
	endfor
End

Macro onscreen() // calls desired graphs and tables
	coeffTable()
	ErrorPlot()
	nyquisterror()
	realandimag()
	nyquist()
	bodephzmod()
End

Function Loadfolder()
	wave algsetupvals
	wave/t testsetupvals
	
End

Function/S getholdstring() //converts hold list of 0's and 1's into a "holdstring" needed by gencurvefit.
	variable ii=0, numheld = numpnts(heldlist)
	wave heldlist
	string holdstr = num2istr(heldlist[0])
	for(ii = 1 ; ii < numheld ; ii += 1)
		holdstr += num2istr(heldlist[ii])
	endfor
	return holdstr
End

Function/s makelegendstring() //makes a string to feed the legend
	wave/t coeffnames
	variable ii,cnum=numpnts(coeffnames)
	string legendstring = "\\s(pars)"+coeffnames[0]+"\r"
	for(ii = 1 ; ii < cnum ; ii += 1)
		legendstring += "\\s(pars#"+num2istr(ii)+")"+coeffnames[ii]+"\r"
	endfor
	return legendstring
End

Function appendrest() //appends the rest of the variable curves to the graph.
	wave/t coeffnames
	variable ii,cnum=numpnts(coeffnames)
	for(ii = 1 ; ii < cnum ; ii += 1)
		AppendToGraph pars[ii][*]
		Execute "ModifyGraph lsize(pars#"+num2istr(ii)+")=2"
	endfor
end

Function simulateFunc() //calculates the function in cartesian and polar systems, as well as the polar residuals.  The cartesian residuals are automatically calculated by gencurvefit.
	variable/g midpointt
	wave realdata,imagdata, coefs, fit_phase, fit_modulus, freqdataa,res_phase,res_modulus,phasedata,modulusdata,res_real,res_imag
	wave/t testsetupvals
	Execute testsetupvals[0]+"(coefs,displayfit,freqdataa)"
	duplicate/o/R=(0,midpointt-1) displayfit, fit_real
	duplicate/o/R=(midpointt, ) displayfit, fit_imag
	fit_phase=180/(Pi)*atan(fit_imag/fit_real)
	fit_modulus=(fit_imag^2+fit_real^2)^(1/2)
	res_phase=phasedata-fit_phase
	res_modulus=modulusdata-fit_modulus
	res_real= realdata-fit_real
	res_imag = imagdata-fit_imag
End

Function loadsimdata()
	wave fit_real,fit_imag, sim_real, sim_imag
	wave/t testsetupvals
	if (stringmatch(testsetupvals[2],"sim_real"))
		testsetupvals[2]="z1data"
		testsetupvals[3]="z2data"
	else
		duplicate/o fit_real sim_real
		duplicate/o fit_imag sim_imag
		testsetupvals[2]="sim_real"
		testsetupvals[3]="sim_imag"
	endif
End

Function UPDT(currentbestfit,population,costmap,updatetime)
	wave currentbestfit,population,costmap
	variable updatetime
	if(updatetime & 2^3) //check if bit 3 is set, indicating that one generation has passed.
		variable/g iii, midpointt
		wave pars, chi2prog, coefs, algsetupvals
		wave/t testsetupvals
		chi2prog[iii]= wavemin(costmap)
		pars[][iii] = coefs[p] 
		if (algsetupvals[13]==1)
			Execute testsetupvals[6]+"()"
		endif
		iii+=1
		simulateFunc()
	
	endif
	return 0
End

Function DAfit() //Do the fit
	variable/g currentii,iii=0
	wave/d res_modulus, res_phase, limits, coefs, algsetupvals, imagdata, realdata,freqdata,freqdataa,fit_phase,fit_modulus,modulusdata,phasedata,fit_real,fit_imag
	wave/t testsetupvals
	variable/g midpointt
	string/g currentmodel
	testsetupvals[0]=currentmodel
	variable/c/g ki=cmplx(0,1)
	variable/c/g kii=cmplx(1,-1)
	string holdstr = getholdstring()
		midpointt=numpnts(freqdata)
//		make/c/d/o/n=(midpointt) complexer
 		make/n=(algsetupvals[6])/d/o Chi2prog = 0
 		SetScale/P x 0,algsetupvals[2]*numpnts(coefs),"", Chi2prog
		make/d/n=(numpnts(coefs), algsetupvals[6])/o pars = 0
		SetScale/P x 0,algsetupvals[2],"", pars
 		Execute "duplicate/o "+ testsetupvals[2] + " realdata" //load data into toaster
		Execute "duplicate/o "+ testsetupvals[3] + " imagdata"
		Execute "duplicate/o "+ testsetupvals[4] + " freqdata"
		concatenate/np/o/d {freqdata,freqdata}, freqdataa
		concatenate/np/o/d {realdata,imagdata}, combdataa
		make/o/d/n=(numpnts(freqdata)) ratiodata
		ratiodata=realdata/imagdata
		make/o/d/n=(numpnts(freqdata)) modulusdata
		duplicate/o modulusdata fit_modulus
		make/d/o/n=(numpnts(freqdata)) phasedata
		duplicate/o phasedata fit_phase
		modulusdata=(realdata^2+imagdata^2)^(1/2)
		duplicate/o realdata res_real
		duplicate/o imagdata res_imag
		duplicate/o modulusdata res_modulus
		phasedata=180/(Pi)*atan(imagdata/realdata)
		duplicate/o phasedata res_phase
		make/d/o/n=(numpnts(freqdataa)) displayfit		

		Execute/q "Gencurvefit/UPDT=UPDT/POL/METH=(algsetupvals[0])/L=(2*midpointt)/R/TOL=(algsetupvals[9])/q=(algsetupvals[1])/K={algsetupvals[6],algsetupvals[2],algsetupvals[3],algsetupvals[4]}/SEED=(algsetupvals[5])/STGY=(algsetupvals[7])/X=freqdataa "+testsetupvals[0]+",combdataa,coefs,\""+getholdstring()+"\",limits"		// do the fit
End

//Tables
Window coeffTable() : Table
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Edit/W=(22,44,527,251) limits,coefs,coeffnames,heldlist
	ModifyTable format(Point)=1
	SetDataFolder fldrSav0
EndMacro

Window algsetup() : Table
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Edit/W=(5,44,511,311) algsetupnames,algsetupvals,testsetupnames,testsetupvals
	ModifyTable format(Point)=1
	SetDataFolder fldrSav0
EndMacro

//Graphs
Window ErrorPlot() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(525,580,842,816) Chi2prog
	SetDataFolder fldrSav0
	ModifyGraph mode=6
	ModifyGraph lSize=2
	ModifyGraph rgb=(1,4,52428)
	ModifyGraph grid=1
	ModifyGraph log(left)=1
	Label left "Error"
	Label bottom "Number of Function Evaluations"
EndMacro

Macro timegraph() : Graph  //produces the realtime graph for coefficient values
	PauseUpdate; Silent 1		// building window...
	variable ii, cnum = numpnts(coefs)
	string parslist = "pars[1][*]"
	PauseUpdate; Silent 1		// building window...
	Display /W=(844,579,1439,817) pars[0][*]
	ModifyGraph lsize(pars)=2
	Legend/C/N=coefficientLegend/J/H={0,2,10} makelegendstring();DelayUpdate
	ModifyGraph mode=6
	ModifyGraph log(left)=1
	appendrest()
	ShowKBColorizePanel()
End

Window ratiocheck() : Graph
	PauseUpdate; Silent 1		// building window...
	Display /W=(951,535.25,1253.25,716.75) ratiodata vs freqdata
	ModifyGraph mode=2
	ModifyGraph lSize=2
	ModifyGraph rgb=(0,26112,13056)
	Label left "Ratio Z1/Z2"
	Label bottom "log(frequency)"
EndMacro

Window realandimag() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(512,292,1046,556) realdata,imagdata vs freqdata
	AppendToGraph fit_real,fit_imag vs freqdata
	AppendToGraph res_real,res_imag vs freqdata
	AppendToGraph res_modulus vs freqdata
	SetDataFolder fldrSav0
	ModifyGraph mode(realdata)=2,mode(imagdata)=2,mode(res_real)=2,mode(res_imag)=2
	ModifyGraph mode(res_modulus)=2
	ModifyGraph lSize(realdata)=2,lSize(imagdata)=2
	ModifyGraph rgb(imagdata)=(0,0,65280),rgb(fit_real)=(52428,17472,1),rgb(fit_imag)=(0,43690,65535)
	ModifyGraph rgb(res_real)=(65535,32768,32768),rgb(res_imag)=(13112,0,26214),rgb(res_modulus)=(19729,1,39321)
	ModifyGraph grid=1
	ModifyGraph log(bottom)=1
	ModifyGraph tick(bottom)=2
	ModifyGraph minor(bottom)=1
	ModifyGraph gridStyle(bottom)=2
	Label left "Impedance (Ohms)"
	Label bottom "Angular Frequency (2PiHz)"
EndMacro

Window bodephzmod() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(510,44,1044,269) modulusdata vs freqdata
	AppendToGraph/R phasedata vs freqdata
	AppendToGraph/R fit_phase vs freqdata
	AppendToGraph fit_modulus vs freqdata
	AppendToGraph/R res_phase vs freqdata
	SetDataFolder fldrSav0
	ModifyGraph mode(modulusdata)=2,mode(phasedata)=2,mode(res_phase)=2
	ModifyGraph lSize(modulusdata)=2,lSize(phasedata)=2
	ModifyGraph rgb(modulusdata)=(19712,0,39168),rgb(phasedata)=(0,39168,0),rgb(fit_phase)=(0,52224,0)
	ModifyGraph rgb(fit_modulus)=(19712,0,39168),rgb(res_phase)=(16386,65535,16385)
	ModifyGraph grid(left)=1,grid(bottom)=1
	ModifyGraph log(left)=1,log(bottom)=1
	ModifyGraph tick(bottom)=2
	Label left "Modulus |Z|"
	Label bottom "Angular Frequency (2PiHz)"
	Label right "Phase Angle, degrees"
	SetAxis right -90,90
EndMacro

Window nyquist() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(1041.75,37.25,1438.5,283.25) imagdata vs realdata
	AppendToGraph fit_imag vs fit_real
	SetDataFolder fldrSav0
	ModifyGraph mode(imagdata)=2
	ModifyGraph lSize(imagdata)=2
	ModifyGraph rgb(imagdata)=(39168,39168,0)
	ModifyGraph zColor(imagdata)={:ellis:onDeck:freqdata,*,*,Rainbow},zColor(fit_imag)={:ellis:onDeck:freqdata,*,*,Rainbow}
	ModifyGraph logZColor=1
	ModifyGraph grid=1
	Label left "Z'' Imaginary Impedance (Ohms)"
	Label bottom "Z' Real Impedance (Ohms)"
	SetAxis/A/R left
EndMacro

Window residuals() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(986,401,1251,506) res_combdataa[0,80],res_combdataa[81,161] vs freqdata
	SetDataFolder fldrSav0
	ModifyGraph mode=2
	ModifyGraph lSize=2
	ModifyGraph rgb(res_combdataa#1)=(0,0,65280)
	ModifyGraph log(bottom)=1
EndMacro

Window dataTable() : Table
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Edit/W=(325,323,830,530) wdata,z1data,z2data,sim_real,sim_imag,weight,mask
	ModifyTable format(Point)=1
	SetDataFolder fldrSav0
EndMacro

Window nyquisterror() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(1044,313,1440,556) res_imag vs res_real
	SetDataFolder fldrSav0
	ModifyGraph gbRGB=(26214,26214,26214)
	ModifyGraph mode=2
	ModifyGraph lSize=4
	ModifyGraph zColor(res_imag)={:ellis:onDeck:freqdata,*,*,Rainbow}
	ModifyGraph logZColor=1
	Label left "-Z'' Residuals"
	Label bottom "Z' Residuals"
	SetAxis/A/R left
EndMacro


Window Graph0() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(844,579,1439,817) pars[0][*],pars[1][*],pars[2][*],pars[3][*],pars[4][*]
	AppendToGraph pars[5][*],pars[6][*]
	SetDataFolder fldrSav0
	ModifyGraph mode(pars)=6
	ModifyGraph log(left)=1
	Legend/C/N=coefficientLegend/J/H={0,2,10} "\\s(pars)RS\r\\s(pars#1)Q1\r\\s(pars#2)R1\r\\s(pars#3)Q2\r\\s(pars#4)R2\r\\s(pars#5)n1\r\\s(pars#6)n2\r"
EndMacro

Window ErrorPlotshow() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(52.5,261.5,516,633.5) chi2Zqandles2Method0Pop30x8m8[*][0],chi2Zqandles2Method0Pop30x8m8[*][1]
	AppendToGraph chi2Zqandles2Method0Pop30x8m8[*][2],chi2Zqandles2Method0Pop30x8m8[*][3]
	AppendToGraph chi2Zqandles2Method0Pop30x8m8[*][4],chi2Zqandles2Method0Pop30x8m8[*][5]
	AppendToGraph chi2Zqandles2Method0Pop30x8m8[*][6],chi2Zqandles2Method0Pop30x8m8[*][7]
	AppendToGraph chi2Zqandles2Method0Pop30x8m8[*][8],chi2Zqandles2Method0Pop30x8m8[*][9]
	AppendToGraph chi2ZqandlesTMethod0Pop30x8m8[*][0],chi2ZqandlesTMethod0Pop30x8m8[*][1]
	AppendToGraph chi2ZqandlesTMethod0Pop30x8m8[*][2],chi2ZqandlesTMethod0Pop30x8m8[*][3]
	AppendToGraph chi2ZqandlesTMethod0Pop30x8m8[*][4],chi2ZqandlesTMethod0Pop30x8m8[*][5]
	AppendToGraph chi2ZqandlesTMethod0Pop30x8m8[*][6],chi2ZqandlesTMethod0Pop30x8m8[*][7]
	AppendToGraph chi2ZqandlesTMethod0Pop30x8m8[*][8],chi2ZqandlesTMethod0Pop30x8m8[*][9]
	AppendToGraph chi2ZqandlesWMethod0Pop30x8m8[*][0],chi2ZqandlesWMethod0Pop30x8m8[*][1]
	AppendToGraph chi2ZqandlesWMethod0Pop30x8m8[*][2],chi2ZqandlesWMethod0Pop30x8m8[*][3]
	AppendToGraph chi2ZqandlesWMethod0Pop30x8m8[*][4],chi2ZqandlesWMethod0Pop30x8m8[*][5]
	AppendToGraph chi2ZqandlesWMethod0Pop30x8m8[*][6],chi2ZqandlesWMethod0Pop30x8m8[*][7]
	AppendToGraph chi2ZqandlesWMethod0Pop30x8m8[*][8],chi2ZqandlesWMethod0Pop30x8m8[*][9]
	AppendToGraph chi2ZqandlesBMethod0Pop30x8m8[*][0],chi2ZqandlesBMethod0Pop30x8m8[*][1]
	AppendToGraph chi2ZqandlesBMethod0Pop30x8m8[*][2],chi2ZqandlesBMethod0Pop30x8m8[*][3]
	AppendToGraph chi2ZqandlesBMethod0Pop30x8m8[*][4],chi2ZqandlesBMethod0Pop30x8m8[*][5]
	AppendToGraph chi2ZqandlesBMethod0Pop30x8m8[*][6],chi2ZqandlesBMethod0Pop30x8m8[*][7]
	AppendToGraph chi2ZqandlesBMethod0Pop30x8m8[*][8],chi2ZqandlesBMethod0Pop30x8m8[*][9]
	SetDataFolder fldrSav0
	ModifyGraph rgb(chi2Zqandles2Method0Pop30x8m8)=(0,0,39168),rgb(chi2Zqandles2Method0Pop30x8m8#1)=(0,0,39168)
	ModifyGraph rgb(chi2Zqandles2Method0Pop30x8m8#2)=(0,0,39168),rgb(chi2Zqandles2Method0Pop30x8m8#3)=(0,0,39168)
	ModifyGraph rgb(chi2Zqandles2Method0Pop30x8m8#4)=(0,0,39168),rgb(chi2Zqandles2Method0Pop30x8m8#5)=(0,0,39168)
	ModifyGraph rgb(chi2Zqandles2Method0Pop30x8m8#6)=(0,0,39168),rgb(chi2Zqandles2Method0Pop30x8m8#7)=(0,0,39168)
	ModifyGraph rgb(chi2Zqandles2Method0Pop30x8m8#8)=(0,0,39168),rgb(chi2Zqandles2Method0Pop30x8m8#9)=(0,0,39168)
	ModifyGraph rgb(chi2ZqandlesTMethod0Pop30x8m8)=(0,52224,0),rgb(chi2ZqandlesTMethod0Pop30x8m8#1)=(0,52224,0)
	ModifyGraph rgb(chi2ZqandlesTMethod0Pop30x8m8#2)=(0,52224,0),rgb(chi2ZqandlesTMethod0Pop30x8m8#3)=(0,52224,0)
	ModifyGraph rgb(chi2ZqandlesTMethod0Pop30x8m8#4)=(0,52224,0),rgb(chi2ZqandlesTMethod0Pop30x8m8#5)=(0,52224,0)
	ModifyGraph rgb(chi2ZqandlesTMethod0Pop30x8m8#6)=(0,52224,0),rgb(chi2ZqandlesTMethod0Pop30x8m8#7)=(0,52224,0)
	ModifyGraph rgb(chi2ZqandlesTMethod0Pop30x8m8#8)=(0,52224,0),rgb(chi2ZqandlesTMethod0Pop30x8m8#9)=(0,52224,0)
	ModifyGraph rgb(chi2ZqandlesWMethod0Pop30x8m8)=(65535,21845,0),rgb(chi2ZqandlesWMethod0Pop30x8m8#1)=(65535,21845,0)
	ModifyGraph rgb(chi2ZqandlesWMethod0Pop30x8m8#2)=(65535,21845,0),rgb(chi2ZqandlesWMethod0Pop30x8m8#3)=(65535,21845,0)
	ModifyGraph rgb(chi2ZqandlesWMethod0Pop30x8m8#4)=(65535,21845,0),rgb(chi2ZqandlesWMethod0Pop30x8m8#5)=(65535,21845,0)
	ModifyGraph rgb(chi2ZqandlesWMethod0Pop30x8m8#6)=(65535,21845,0),rgb(chi2ZqandlesWMethod0Pop30x8m8#7)=(65535,21845,0)
	ModifyGraph rgb(chi2ZqandlesWMethod0Pop30x8m8#8)=(65535,21845,0),rgb(chi2ZqandlesWMethod0Pop30x8m8#9)=(65535,21845,0)
	ModifyGraph grid=1
	ModifyGraph log(left)=1
	Label left "X\\S2 \\MError"
	Label bottom "Number of Function Evaluations"
	Legend/C/N=text0/J/H={0,3,10}/A=MB/X=6.27/Y=64.41 "\\s(chi2Zqandles2Method0Pop30x8m8) 2 QPE\r\\s(chi2ZqandlesTMethod0Pop30x8m8) T QPE"
	AppendText "\\s(chi2ZqandlesWMethod0Pop30x8m8) W QPE\r\\s(chi2ZqandlesBMethod0Pop30x8m8) B QPE\r"
EndMacro

Window errorplot0() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:ellis:onDeck:
	Display /W=(36,435.5,430.5,644) chi2stack[*][0],chi2stack[*][0],chi2stack[*][1],chi2stack[*][2]
	AppendToGraph chi2stack[*][3],chi2stack[*][4],chi2stack[*][5],chi2stack[*][6],chi2stack[*][7]
	AppendToGraph chi2stack[*][8],chi2stack[*][9]
	SetDataFolder fldrSav0
	ModifyGraph log(left)=1
EndMacro


Function stackError(trials,conditions,currentiiii)
	variable trials, conditions, currentiiii
	variable iiii
	wave limitstack,teststack,algsetupvals, chi2prog, pars,coefs
	wave/t testsetupvals
	if (currentiiii==0)
		make/n=(numpnts(chi2prog),trials)/d/o chi2stack
		make/n=(numpnts(coefs),numpnts(chi2prog),trials)/d/o parsStack
		make/n=(numpnts(coefs),trials)/d/o resultStack
	endif
	for(iiii = currentiiii; iiii < trials ; iiii += 1)
		DAfit()
		chi2stack[ ][iiii]=chi2prog[p]
		parsStack[ ][ ][iiii]=pars[p][p]
		resultStack[ ][iiii]=coefs[p]
		AppendToGraph/W=ErrorPlot/T chi2stack[ ][iiii]
		algsetupvals[11]=iiii
	endfor
	Execute "duplicate/o chi2stack, chi2"+(testsetupvals[0])+"Method"+num2istr(algsetupvals[0])+"Pop"+num2istr(algsetupvals[2])+"x"+num2istr(10*algsetupvals[3])+"m"+num2istr(10*algsetupvals[4])
	Execute "duplicate/o parsStack, pars"+(testsetupvals[0])+"Method"+num2istr(algsetupvals[0])+"Pop"+num2istr(algsetupvals[2])+"x"+num2istr(10*algsetupvals[3])+"m"+num2istr(10*algsetupvals[4])

End

Function chi2plotter(currentiiii,trials)
	variable currentiiii, trials
	variable iiii
	for(iiii = currentiiii; iiii < trials ; iiii += 1)
		AppendToGraph root:ellis:onDeck:chi2ZqandlesBMethod0Pop30x8m8[ ][iiii]
	endfor
End

Function appenchi2stack(stackname)
	string stackname
	variable iiii
	for (iiii=0;iiii<10;iiii+=1)
		Execute "AppendToGraph "+stackname+"[ ][iiii]"
	endfor
End

Function RunConfidenceBands()
	variable ninput
	Prompt ninput, "n-trials:"		// Set prompt for x param
	DoPrompt "How many monte carlo trials would you like to perform for your confidence band?",ninput
	ConfidenceBands(ninput)
	simulatefunc()
End

Function ConfidenceBands(numtrials)//basecoefs,ranger,numtrials)
	variable numtrials
	wave  W_sigma,coefs	
	variable numcoefs = numpnts(coefs)
	make/o/n=((numcoefs),(numtrials)) montecarlomatrix
	montecarlomatrix[ ][ ]=enoise(W_sigma[p])+coefs[p]
	variable/g midpointt
	make/o/n=(midpointt,2,numtrials) montecarloresults
	wave fit_real, fit_imag
	wave freqdataa, displayfit
	wave/t testsetupvals
	duplicate/o coefs tempcoef
//	wave tempcoef
	variable ii
	for (ii = 0; ii<numtrials;ii+=1)
		tempcoef = montecarlomatrix[p][ii]
//		print tempcoef
		Execute testsetupvals[0]+"(tempcoef,displayfit,freqdataa)"
		duplicate/o/R=(0,midpointt-1) displayfit, fit_real
		duplicate/o/R=(midpointt, ) displayfit, fit_imag
		montecarloresults[ ][0][ii]=fit_real[p]//displayfit[0, midpointt-1]//
		montecarloresults[ ][1][ii]=fit_imag[p]//displayfit[midpointt, ]//
	endfor	
	duplicate/o fit_real real_max
	duplicate/o fit_real real_min
	duplicate/o fit_imag imag_max
	duplicate/o fit_imag imag_min
	make/o/n=(numtrials) trialwave
	
	for (ii = 0; ii<numtrials;ii+=1)
		trialwave=montecarloresults[ii][0][p]
		real_max[ii]=wavemax(trialwave)
		real_min[ii]=wavemin(trialwave)
		trialwave=montecarloresults[ii][1][p]
		imag_max[ii]=wavemax(trialwave)
		imag_min[ii]=wavemin(trialwave)
	endfor
End

Function displayconfidenceedges()
	Execute "nyquist()"
	Appendtograph imag_max vs real_max
	appendtograph imag_min vs real_min
	Execute "realandimag()"
	appendtograph imag_max vs freqdata
	appendtograph imag_min vs freqdata
	appendtograph real_max vs freqdata
	appendtograph real_min vs freqdata
End

Function calculatebands()
	variable numtrials, ii
	wave montecarloresults, fit_real, fit_imag

End

Function RunDisplayConfidenceBands()
	variable ninput
	Prompt ninput, "n-trials to display:"		// Set prompt for x param
	DoPrompt "How many trials would you like to display for your confidence band?",ninput
	Execute "nyquist()"
	DisplayConfidencebandsnyquist(ninput)
	Execute "realandimag()"
	Displayconfidencebandsbode(ninput)
	
End

Function Displayconfidencebandsbode(numtrials)
	variable numtrials
	variable ii
	for(ii = 0 ; ii < numtrials ; ii += 1)
		AppendToGraph montecarloresults[ ][1][ii] vs freqdata
		AppendToGraph montecarloresults[ ][0][ii] vs freqdata
	endfor
end

Function Displayconfidencebandsnyquist(numtrials)
	variable numtrials
	variable ii
	for(ii = 0 ; ii < numtrials ; ii += 1)
		AppendToGraph montecarloresults[ ][1][ii] vs montecarloresults[ ][0][ii]
	endfor
end